package enseirb.myinpulse;

import static enseirb.myinpulse.model.ProjectDecisionValue.*;

import static org.junit.jupiter.api.Assertions.*;

import enseirb.myinpulse.model.Entrepreneur;
import enseirb.myinpulse.model.Project;
import enseirb.myinpulse.model.SectionCell;
import enseirb.myinpulse.service.EntrepreneurApiService;
import enseirb.myinpulse.service.database.EntrepreneurService;
import enseirb.myinpulse.service.database.ProjectService;
import enseirb.myinpulse.service.database.SectionCellService;

import org.junit.jupiter.api.BeforeAll;
import org.junit.jupiter.api.Test;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.boot.test.context.SpringBootTest;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.web.server.ResponseStatusException;

import java.time.LocalDate;
import java.time.LocalDateTime;
import java.util.ArrayList;
import java.util.List;

@SpringBootTest
@Transactional
public class EntrepreneurApiServiceTest {
    private static Entrepreneur entrepreneur;
    private static Project project;
    private static Iterable<SectionCell> sectionCells;
    @Autowired private EntrepreneurApiService entrepreneurApiService;
    @Autowired private EntrepreneurService entrepreneurService;
    @Autowired private ProjectService projectService;
    @Autowired private SectionCellService sectionCellService;

    @BeforeAll
    static void setup(
            @Autowired EntrepreneurService entrepreneurService,
            @Autowired ProjectService projectService,
            @Autowired SectionCellService sectionCellService) {
        entrepreneur =
                entrepreneurService.addEntrepreneur(
                        new Entrepreneur(
                                "entre",
                                "preneur",
                                "entrepreneur@mail.fr",
                                "entrepreneur2@mail.fr",
                                "01 45 71 25 48",
                                "ENSEIRB",
                                "Info",
                                false));
        entrepreneurService.addEntrepreneur(
                new Entrepreneur(
                        "entre2",
                        "preneur2",
                        "testentrepreneur@mail.fr",
                        "testentrepreneur2@mail.fr",
                        "",
                        "ENSEGID",
                        "",
                        true));
        project =
                projectService.addNewProject(
                        new Project("Project", null, LocalDate.now(), ACTIVE, null, entrepreneur));
        // projectService.updateProjectEntrepreneurParticipation(project.getIdProject(),
        // entrepreneur); proxy error because why not
        entrepreneurService.updateEntrepreneurProjectProposed(entrepreneur.getIdUser(), project);
        entrepreneurService.updateEntrepreneurProjectParticipation(
                entrepreneur.getIdUser(), project);
        System.out.println(("real"));
        System.out.println(entrepreneur);
        // System.out.println(entrepreneur.getProjectProposed());
        // System.out.println(entrepreneur.getProjectParticipation());
        sectionCellService.addNewSectionCell(
                new SectionCell(
                        null, 2L, "contenu très intéressant", LocalDateTime.now(), project));
        sectionCells = sectionCellService.getSectionCellsByProject(project, 2L);
    }

    private <T> List<T> IterableToList(Iterable<T> iterable) {
        List<T> l = new ArrayList<>();
        iterable.forEach(l::add);
        return l;
    }

    @Test
    void editValidSectionCell() {
        System.out.println("START\n\n\n");
        entrepreneurApiService.editSectionCell(
                IterableToList(sectionCells).getFirst().getIdSectionCell(),
                "modified content",
                "entrepreneur@mail.fr");
        // We get the data from the database again.
        sectionCells = sectionCellService.getSectionCellsByProject(project, 2L);
        assertEquals(
                "modified content",
                IterableToList(sectionCells).getFirst().getContentSectionCell());
        System.out.println("END\n\n\n");
    }

    @Test
    void editInvalidSectionCell() {
        assertThrows(
                ResponseStatusException.class,
                () ->
                        entrepreneurApiService.editSectionCell(
                                -1L, "should not be modified", "entrepreneur@mail.fr"));
    }

    @Test
    void editSectionCellInvalidAccess() {
        assertThrows(
                ResponseStatusException.class,
                () ->
                        entrepreneurApiService.editSectionCell(
                                IterableToList(sectionCells).getFirst().getIdSectionCell(),
                                "should not be modified",
                                "testentrepreneur@mail.fr"));
        assertEquals(
                "contenu très intéressant",
                IterableToList(sectionCells).getFirst().getContentSectionCell());
    }

    @Test
    void removeValidSectionCell() {
        SectionCell tmpCell =
                sectionCellService.addNewSectionCell(
                        new SectionCell(
                                null, 2L, "contenu temporaire", LocalDateTime.now(), project));
        assertEquals(
                2, IterableToList(sectionCellService.getSectionCellsByProject(project, 2L)).size());
        entrepreneurApiService.removeSectionCell(
                tmpCell.getIdSectionCell(), "entrepreneur@mail.fr");
        assertEquals(
                1, IterableToList(sectionCellService.getSectionCellsByProject(project, 2L)).size());
    }

    @Test
    void removeInvalidSectionCell() {
        assertThrows(
                ResponseStatusException.class,
                () -> entrepreneurApiService.removeSectionCell(-1L, "entrepreneur@mail.fr"));
    }
}
